﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UPP.OGUI;
using UnityEditor;

namespace UPP.OGUI
{

    public abstract class Inline : Panel
    {
        private bool _autoSize = true;
        public bool AutoSize { get { return _autoSize; } set { SetAutosize(value); } }

        private GUIStyle _style;

        public GUIStyle GetCurrentStyle() { return _style; }

        private void Init()
        {
            _style = DefaultStyle();
            SetSizeFromContent();
        }

        public Inline()
        {
            Init();
        }

        abstract public GUIContent GetContent();
        abstract public GUIStyle DefaultStyle();

        protected void SetSizeFromContent()
        {
            Vector2 v = _style.CalcSize(new GUIContent(GetContent()));
            this.Size = new UPP.Size(v.x + _style.padding.left + _style.padding.right
                + _style.margin.left + _style.margin.right,
                v.y + _style.padding.top + _style.padding.bottom
                + _style.margin.top + _style.margin.bottom);

            _style.wordWrap = !AutoSize;
        }

        private void SetAutosize(bool newVal)
        {
            if (newVal != _autoSize)
            {
                _autoSize = newVal;
                SetSizeFromContent();
            }
        }
    }

}