﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEditor;
using UnityEngine;

namespace UPP.OGUI
{
    public class NumBox : Inline
    {

        private int _value = 0;
        public int Value { get { return _value; } set { ChangeValue(value); } }

        public delegate void OnChangeHandler(object sender);
        public event OnChangeHandler OnChange;

        int _minValue = 0;
        int _maxValue = 100000;

        public int MinValue { get { return _minValue; } set { SetMin(value); } }
        public int MaxValue { get { return _maxValue; } set { SetMax(value); } }

        private void SetMin(int min)
        {
            _minValue = min;
            if (_maxValue < _minValue)
                _maxValue = _minValue;
            _value = ValidVal(_value);
        }

        private void SetMax(int max)
        {
            _maxValue = max;
            if (_maxValue < _minValue)
                _minValue = _maxValue;
            _value = ValidVal(_value);
        }

        private int ValidVal(int val)
        {
            return Math.Max(Math.Min(val, _maxValue), _minValue);
        }

        private void ChangeValue(int newVal)
        {
            if (newVal != _value)
            {
                _value = ValidVal(newVal);
                if (AutoSize)
                    SetSizeFromContent();

                if (OnChange != null)
                    OnChange(this);
            }
        }

        private void Init(int value, int min, int max)
        {
            _minValue = min;
            _maxValue = max;
            _value = ValidVal(value);
            this.InheritedContent = DrawInherited;
            SetSizeFromContent();
            this.AutoSize = false;
            this.Size = new UPP.Size(150, GetCurrentStyle().CalcHeight(new GUIContent(), 150)
                + GetCurrentStyle().padding.top + GetCurrentStyle().padding.bottom);
        }


        private void DrawInherited()
        {
            if (AutoSize)
            {
                int s = EditorGUILayout.IntField(_value, GetCurrentStyle());

                if (s != _value)
                    this.Value = ValidVal(s); // Also will trigger the event.
            }
            else
            {
                int s = EditorGUILayout.IntField(_value, GetCurrentStyle(),
                    GUILayout.Width(this.Size.Width - GetCurrentStyle().padding.left - GetCurrentStyle().padding.right),
                    GUILayout.Height(this.Size.Height - GetCurrentStyle().padding.top - GetCurrentStyle().padding.bottom)
                    );

                if (s != _value)
                    this.Value = ValidVal(s); // Also will trigger the event.
            }
        }

        public override GUIContent GetContent()
        {
            return new GUIContent(_value.ToString());
        }

        public override GUIStyle DefaultStyle()
        {
            GUIStyle skin = new GUIStyle(EditorStyles.numberField);
            skin.margin = new RectOffset(4, 0, 1, 0);
            skin.padding = new RectOffset(5, 5, 1, 1);
            return new GUIStyle(skin);
        }

        public NumBox()
        {
            Init(0, 0, 100000);
        }

        public NumBox(int value)
        {
            Init(value, 0, 100000);
        }

        public NumBox(int value, int min, int max)
        {
            Init(value, min, max);
        }
    }
}