﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEditor;
using UnityEngine;

namespace UPP.OGUI
{

    public class TabButtons : Inline
    {
        private List<string> _buttons = new List<string>();
        int _selected = 0;

        public int Selected { get { return _selected; } }

        const int lineHeight = 18;

        public delegate void OnSelectionChangekHandler(object sender, int id, string text);
        public event OnSelectionChangekHandler OnSelectionChange;

        GUIStyle backdropSkin;

        public void Select(int s)
        {
            if (_selected == s)
                return;

            if (s < 0 || s >= _buttons.Count)
                return;

            _selected = s;
            if (OnSelectionChange != null)
                OnSelectionChange(this, _selected, _buttons[_selected]);
        }

        public void SetButtons(IList<string> buttons)
        {
            _buttons = new List<string>(buttons);
            RecomputeBackdrop();
        }

        public void RecomputeBackdrop()
        {
            float per = Size.Width / _buttons.Count;
            backdropSkin = new GUIStyle(GUIStyle.none);
            int w = (int)per;
            int h = 3;
            if (w > 0 && h > 0)
            {
                Texture2D t = new Texture2D(w, h);
                ColorSchemes.FillTexture(t, ColorSchemes.SelectionColor);
                backdropSkin.normal.background = t;
            }
        }

        private void Init(IList<string> buttons, int selected)
        {
            if (buttons != null)
                SetButtons(buttons);
            Select(selected);
            this.InheritedContent = DrawInherited;
            SetSizeFromContent();
            RecomputeBackdrop();
            this.BackgroundColor = ColorSchemes.BackgroundColorDarker;
        }

        public TabButtons(IList<string> buttons = null, int selected = 0)
        {
            Init(buttons, selected);
        }

        public override GUIStyle DefaultStyle()
        {
            GUIStyle skin = new GUIStyle(EditorStyles.toolbarButton);
            skin.margin = new RectOffset(0, 0, 0, 0);
            return new GUIStyle(skin);
        }

        public override GUIContent GetContent()
        {
            return new GUIContent(String.Join(", ", _buttons.ToArray()));
        }

        private void DrawInherited()
        {
            int s = _selected;
            if (AutoSize)
            {
                s = GUILayout.Toolbar(_selected, _buttons.ToArray(), GetCurrentStyle());
            }
            else
            {
                s = GUILayout.Toolbar(_selected, _buttons.ToArray(), GetCurrentStyle(),
                        GUILayout.Width(this.Size.Width - GetCurrentStyle().padding.left - GetCurrentStyle().padding.right),
                        GUILayout.Height(this.Size.Height - GetCurrentStyle().padding.top - GetCurrentStyle().padding.bottom),
                        GUILayout.MinHeight(this.Size.Height - GetCurrentStyle().padding.top - GetCurrentStyle().padding.bottom));

            }
            float per = Size.Width / _buttons.Count;
            GUI.Label(new Rect(s * per, lineHeight, per+1, Size.Height - lineHeight), "", backdropSkin);
            if (s != _selected)
                Select(s);
        }
    }
}