﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using UnityEditor;
using UnityEngine;
using UPP.OGUI;

namespace UPP.Engine
{
    public class ClassEditor : Window
    {
        [MenuItem("Unity++/Class Editor")]
        public static void ShowWindow()
        {
            EditorWindow.GetWindow(typeof(ClassEditor),
                Boolean.Parse(Settings.GetOrSet("UPP_windows_as_utility", true.ToString())),
                "Class Editor");
        }

        List<DiagramEntity> entityDiagrams = new List<DiagramEntity>();
        DiagramEntity lastSelectedDiagram = null;

        TreeView hierarchyTreeView = null;
        Table attributeTable = null;
        Table methodTable = null;

        Label infoLabel = null;
        TabButtons tabButtons = null;

        Panel diagramCanvas = null;
        Panel attributeCanvas = null;
        Panel methodCanvas = null;

        Panel toolsClasses = null;
        Panel toolsAttributes = null;

        int lastSelectedBackupId = -1;

        ClassEditorEdit classEditorEdit = null;
        PropertyTypeWindow propTypeWindow = null;

        protected override void PrepareGUI()
        {
            Panel menu = new Panel();
            menu.Dock = DockStyle.Top;
            menu.Size.Height = 25;
            menu.BorderColor = ColorSchemes.BorderColorDarker;

            Panel info = new Panel();
            info.Dock = DockStyle.Bottom;
            info.Size.Height = 25;
            info.BorderColor = ColorSchemes.BorderColorDarker;

            infoLabel = new Label();
            infoLabel.Parent = info;
            infoLabel.Size.Width = info.Size.Width - 16;
            infoLabel.Anchor = new Anchor(true, true, true, true);
            infoLabel.Position = new Position(8, 5);

            SplitContainer spl = new SplitContainer(SplitContainer.ModeType.Horizontal, 25, SplitContainer.SizeType.Percentage, SplitContainer.PanelWho.First);
            spl.Dock = DockStyle.Fill;

            diagramCanvas = new Panel();
            diagramCanvas.Parent = spl.GetPanel(SplitContainer.PanelWho.Second);
            diagramCanvas.Dock = DockStyle.Fill;
            diagramCanvas.BackgroundColor = ColorSchemes.BackgroundColorDarker;
            diagramCanvas.SizeMode = Panel.SizeModeType.ScrollBars;
            diagramCanvas.IsEntityDiagramCanvas = true;

            attributeCanvas = new Panel();
            attributeCanvas.Parent = spl.GetPanel(SplitContainer.PanelWho.Second);
            attributeCanvas.Dock = DockStyle.Fill;
            attributeCanvas.Size = spl.GetPanel(SplitContainer.PanelWho.Second).Size;
            attributeCanvas.BackgroundColor = ColorSchemes.BackgroundColorDarker;
            attributeCanvas.SizeMode = Panel.SizeModeType.Clip;
            attributeCanvas.Visible = false;

            methodCanvas = new Panel();
            methodCanvas.Parent = spl.GetPanel(SplitContainer.PanelWho.Second);
            methodCanvas.Dock = DockStyle.Fill;
            methodCanvas.Size = spl.GetPanel(SplitContainer.PanelWho.Second).Size;
            methodCanvas.BackgroundColor = ColorSchemes.BackgroundColorDarker;
            methodCanvas.SizeMode = Panel.SizeModeType.Clip;
            methodCanvas.Visible = false;

            Panel hierarchy = new Panel();
            hierarchy.Parent = spl.GetPanel(SplitContainer.PanelWho.First);
            hierarchy.Dock = DockStyle.Fill;

            hierarchyTreeView = new TreeView();
            hierarchyTreeView.Parent = hierarchy;
            hierarchyTreeView.Dock = DockStyle.Fill;
            ReconstructFromData();
            hierarchyTreeView.Refresh();
            hierarchyTreeView.OnSelectionChange += Tview_OnSelectionChange;
            hierarchyTreeView.OnDoubleClick += HierarchyTreeView_OnDoubleClick;
            hierarchyTreeView.ExpandAll();

            tabButtons = new TabButtons(new string[] { "Class Creation", "Attributes", "Methods" }, 0);
            tabButtons.Parent = menu;
            tabButtons.OnSelectionChange += Tab_OnSelectionChange;
            tabButtons.Size = new Size(300, menu.Size.Height);
            tabButtons.BorderColor = ColorSchemes.BorderColor;

            Button bAdd = new Button("Add class");
            bAdd.Anchor = new Anchor(true, true, false, false);
            bAdd.Position = new Position(3, 3);
            bAdd.OnClick += BAdd_OnClick;

            Button bEdit = new Button("Edit class");
            bEdit.Anchor = new Anchor(true, true, false, false);
            bEdit.Position = new Position(bAdd.Position.X + bAdd.Size.Width + 3, 3);
            bEdit.OnClick += BEdit_OnClick;

            Button bRemove = new Button("Remove class");
            bRemove.Anchor = new Anchor(true, true, false, false);
            bRemove.Position = new Position(bEdit.Position.X + bEdit.Size.Width + 3, 3);
            bRemove.OnClick += BRemove_OnClick;

            toolsClasses = new Panel(menu);
            toolsClasses.Size = new Size(bAdd.Size.Width + bRemove.Size.Width + bEdit.Size.Width + 12, menu.Size.Height);
            toolsClasses.Dock = DockStyle.Right;
            bAdd.Parent = toolsClasses;
            bEdit.Parent = toolsClasses;
            bRemove.Parent = toolsClasses;

            attributeTable = new Table(new string[] { "Attribute", "Type", "Configuration", "Description" });
            attributeTable.Parent = attributeCanvas;
            attributeTable.Columns[0].Width = 200;
            attributeTable.Columns[1].Width = 100;
            attributeTable.Columns[2].Width = 200;
            attributeTable.Size.Width = attributeCanvas.Size.Width;
            attributeTable.Anchor = new Anchor(true, true, true, true);
            attributeTable.BackgroundColor = ColorSchemes.BackgroundColor;
            attributeTable.OnDoubleClick += AttributeTable_OnDoubleClick;

            methodTable = new Table(new string[] { "Method Record" });
            methodTable.Parent = methodCanvas;
            methodTable.Size.Width = methodCanvas.Size.Width;
            methodTable.Anchor = new Anchor(true, true, true, true);
            methodTable.BackgroundColor = ColorSchemes.BackgroundColor;

            Button aAdd = new Button("Add attribute");
            aAdd.Anchor = new Anchor(true, true, false, false);
            aAdd.Position = new Position(3, 3);
            aAdd.OnClick += AAdd_OnClick; ;

            Button aEdit = new Button("Edit attribute");
            aEdit.Anchor = new Anchor(true, true, false, false);
            aEdit.Position = new Position(aAdd.Position.X + aAdd.Size.Width + 3, 3);
            aEdit.OnClick += AEdit_OnClick;

            Button aRemove = new Button("Remove attribute");
            aRemove.Anchor = new Anchor(true, true, false, false);
            aRemove.Position = new Position(aEdit.Position.X + aEdit.Size.Width + 3, 3);
            aRemove.OnClick += ARemove_OnClick;

            toolsAttributes = new Panel(menu);
            toolsAttributes.Size = new Size(aAdd.Size.Width + aRemove.Size.Width + aEdit.Size.Width + 12, menu.Size.Height);
            toolsAttributes.Dock = DockStyle.Right;
            toolsAttributes.Visible = false;
            aAdd.Parent = toolsAttributes;
            aEdit.Parent = toolsAttributes;
            aRemove.Parent = toolsAttributes;

            this.OnModalReturn += ClassEditor_OnModalReturn;
        }

        private void ARemove_OnClick(object sender)
        {
            if (attributeTable.SelectedRow == -1)
                return;

            var i = hierarchyTreeView.SelectedItem;
            string pr = attributeTable.Rows[attributeTable.SelectedRow].Tag;

            if (i != null)
            {
                if (PropertyManager.Remove(i.MachineName, pr))
                {
                    LoadAttributes(i.MachineName);
                }
            }
        }

        private void AEdit_OnClick(object sender)
        {
            EditAttributeDialog();
        }

        private void AttributeTable_OnDoubleClick(object sender, int row)
        {
            EditAttributeDialog();
        }

        private void AAdd_OnClick(object sender)
        {
            AddAttributeDialog();
        }

        private void SelectTab(int id)
        {
            tabButtons.Select(id);
        }

        private void LoadMethods(string machine)
        {
            methodTable.Clear();
            var cls = ClassManager.GetClassByMachineName(machine);
            if (cls == null)
                return;

            string s = Settings.GetOrSet("UPP_Output_Namespace", "UPP.Game") +
                "." + machine;

            var methods = Type.GetType(s).GetMethods()
                      .Where(m => m.GetCustomAttributes(typeof(UPP.Engine.MethodInfo), false).Length > 0)
                      .ToArray();

            foreach (var method in methods)
            {
                string par = "";
                foreach (var p in method.GetParameters())
                {
                    if (par != "") par += ", ";
                    par += p.ParameterType.ToString();
                }

                var prInfo = (MethodInfo)Attribute.GetCustomAttribute(method, typeof(MethodInfo));
                if (prInfo == null)
                    continue;

                methodTable.AddRow(new string[] {
                    method.ReturnType.ToString() + " " +
                    method.Name + " (" + par + ") - " + prInfo.Description
                });
            }
        }

        private void Tab_OnSelectionChange(object sender, int id, string text)
        {
            if (id == 0)
            {
                diagramCanvas.Visible = true;
                attributeCanvas.Visible = false;
                methodCanvas.Visible = false;
            }
            if (id == 1)
            {
                diagramCanvas.Visible = false;
                methodCanvas.Visible = false;
                attributeCanvas.Visible = true;
                if (hierarchyTreeView.SelectedItem != null)
                    LoadAttributes(hierarchyTreeView.SelectedItem.MachineName);
                attributeTable.Redraw();
            }
            if (id == 2)
            {
                diagramCanvas.Visible = false;
                attributeCanvas.Visible = false;
                methodCanvas.Visible = true;
                if (hierarchyTreeView.SelectedItem != null)
                    LoadMethods(hierarchyTreeView.SelectedItem.MachineName);
                methodTable.Redraw();
            }

            toolsClasses.Visible = diagramCanvas.Visible;
            toolsAttributes.Visible = attributeCanvas.Visible;
        }

        private void HierarchyTreeView_OnDoubleClick(object sender, string machineName, string displayName, int internalId)
        {
            EditClassDialog();
        }

        private void ClassEditor_OnModalReturn(DialogResult result, Type windType, object wind)
        {
            if (result == DialogResult.OK && windType == typeof(ClassEditorEdit))
            {
                var w = (ClassEditorEdit)wind;
                ReconstructFromData();
                hierarchyTreeView.Refresh();
                if (w.Mode == ClassEditorEdit.EditorEditMode.Edit)
                    hierarchyTreeView.SelectByInternalId(lastSelectedBackupId);
                else
                {
                    if (lastSelectedBackupId != -1)
                        hierarchyTreeView.SetExpanded(lastSelectedBackupId, true);
                    hierarchyTreeView.SelectByMychineName(ClassManager.LastManipulatedClassName);
                }
            }

            if (windType == typeof(PropertyWindowInstanceSettings))
            {
                if (lastAttAddmode == true)
                {
                    AddAttributeDialogFinalize(result);
                }
                else
                {
                    EditAttributeDialogFinalize(result);
                }
            }

            if (result == DialogResult.OK && windType == typeof(PropertyTypeWindow))
            {
                var t = (wind as PropertyTypeWindow).GetSelectedType();
                AddAttributeDialogAfter(t);
            }
        }

        private void BRemove_OnClick(object sender)
        {
            var i = hierarchyTreeView.SelectedItem;
            int parent = -1;
            if (i.Parent != null)
                parent = i.Parent.internalTreeId;
            if (i != null)
            {
                if (ClassManager.Remove(i.MachineName))
                {
                    ReconstructFromData();
                    hierarchyTreeView.Refresh();
                    if (parent != -1)
                        hierarchyTreeView.SelectByInternalId(parent);
                }
            }
        }

        bool selectingClass = false;
        private void SelectClass(string machineName)
        {

            if (selectingClass)
                return;
            selectingClass = true;

            if (machineName == "")
            {
                selectingClass = false;
                infoLabel.Text = "";
                return;
            }

            var cl = ClassManager.GetClassByMachineName(machineName);
            if (cl != null)
            {
                string a = "";
                if (cl.isAbstract)
                    a = "[abstract] ";
                infoLabel.Text = a + cl.className + " (" + cl.machineName + "): " + cl.description;
            }

            hierarchyTreeView.SelectByMychineName(machineName);
            if (lastSelectedDiagram != null)
                lastSelectedDiagram.Selected = false;
            var diagram = entityDiagrams.Find(x => x.MachineName == machineName);
            diagram.Selected = true;
            lastSelectedDiagram = diagram;

            if (tabButtons.Selected == 1)
                LoadAttributes(machineName);
            if (tabButtons.Selected == 2)
                LoadMethods(machineName);

            selectingClass = false;
        }

        private void Tview_OnSelectionChange(object sender, string machineName, string displayName, int internalId)
        {
            SelectClass(machineName);
        }

        private void BAdd_OnClick(object sender)
        {
            AddClassDialog();
        }

        private void ReconstructFromData()
        {
            hierarchyTreeView.Clear();
            List<ClassDBRecord> listToCreate = new List<ClassDBRecord>(ClassManager.DB.database);
            List<KeyValuePair<string, TreeView.TreeViewItem>> parents = new List<KeyValuePair<string, TreeView.TreeViewItem>>();

            listToCreate.Sort((x, y) => (x.className == y.className) ? (x.machineName.CompareTo(y.machineName)): (x.className.CompareTo(y.className)));
            int defPoint = 50;

            foreach (var e in entityDiagrams)
            {
                e.Parent = null;
                e.WindowParent = null;
            }
            entityDiagrams = new List<DiagramEntity>();
            lastSelectedDiagram = null;

            while (listToCreate.Count > 0)
            {
                int failsafe = listToCreate.Count;
                List<ClassDBRecord> list = new List<ClassDBRecord>(listToCreate);
                List<KeyValuePair<string, TreeView.TreeViewItem>> foundParents = new List<KeyValuePair<string, TreeView.TreeViewItem>>();
                foreach (var c in list)
                {
                    var cl = ClassManager.GetClassByMachineName(c.machineName);
                    bool create = false;
                    TreeView.TreeViewItem foundParent = null;
                    if (cl.machineParent == "")
                    {
                        create = true;
                    }
                    else
                    {
                        // Try to find a parent if it's already created.
                        var parent = parents.Find(x => x.Key == cl.machineParent);
                        if (!parent.Equals(default(KeyValuePair<string, TreeView.TreeViewItem>)))
                        {
                            // Parent already exits.
                            foundParent = parent.Value;
                            create = true;
                        }
                    }

                    if (create)
                    {
                        TreeView.TreeViewItem item = new TreeView.TreeViewItem(c.machineName, c.className);
                        hierarchyTreeView.AddItem(item, foundParent);
                        foundParents.Add(new KeyValuePair<string, TreeView.TreeViewItem>(c.machineName, item));
                        listToCreate.Remove(c);

                        string dTit = c.className;
                        if (cl.isAbstract)
                            dTit = "*" + dTit;
                        DiagramEntity d = new DiagramEntity(this, dTit);
                        d.MachineName = c.machineName;
                        d.Parent = diagramCanvas;
                        d.OnClick += D_OnClick;
                        d.OnMouseUp += D_OnMouseUp;
                        d.PositionChanged += D_PositionChanged;
                        d.OnButtonClick += D_OnButtonClick;
                        entityDiagrams.Add(d);
                        if (cl.diagramX == -1 || cl.diagramY == -1)
                        {
                            d.Position = new Position(defPoint, defPoint);
                            defPoint += 25;
                        }
                        else
                        {
                            d.Position = new Position(cl.diagramX, cl.diagramY);
                        }
                        if (foundParent != null)
                            d.EntityParent = entityDiagrams.Find(x => x.MachineName == foundParent.MachineName);
                    }

                }
                parents.AddRange(foundParents);
                if (failsafe == listToCreate.Count)
                {
                    Debug.LogError("Some classes could not be created inside of the hierarchy panel (" + failsafe.ToString() + ")");
                    break;
                }
            }

            foreach (var e in entityDiagrams)
            {
                var c = e.EntityChildren.Count;
                e.SetContent(new string[] {
                    "Parent: " + ((e.EntityParent != null)?e.EntityParent.Title: "None"),
                    "Children: " + ((c > 0)? c.ToString() + " total" : "None")
                });
                e.SetButtons(new string[] {
                    "Edit", "Attributes"
                });
                e.WindowParent = null;
            }
        }

        private void D_OnButtonClick(object sender, int windowId, int buttonIndex, string buttonText)
        {
            if (buttonIndex == 0)
                EditClassDialog();
            if (buttonIndex == 1)
                SelectTab(1);
        }

        private void D_OnMouseUp(object sender, int windowId, bool wasRepositioned)
        {
            if (wasRepositioned)
                ClassManager.ForceSave((sender as DiagramEntity).MachineName);
        }

        private void D_PositionChanged(object sender, float oldX, float oldY, float newX, float newY)
        {
            var d = sender as DiagramEntity;
            var o = ClassManager.GetClassByMachineName(d.MachineName);
            o.diagramX = (int)newX;
            o.diagramY = (int)newY;
        }

        private void D_OnClick(object sender, int windowId)
        {
            SelectClass((sender as DiagramEntity).MachineName);
        }

        private void AddClassDialog()
        {
            if (hierarchyTreeView.SelectedItem != null)
                lastSelectedBackupId = hierarchyTreeView.SelectedItem.internalTreeId;
            else
                lastSelectedBackupId = -1;
            classEditorEdit = EditorWindow.GetWindow(typeof(ClassEditorEdit), true, "Create Class") as ClassEditorEdit;
            ModalChild = classEditorEdit;
            if (hierarchyTreeView.SelectedItem != null)
                classEditorEdit.PreSetParent = hierarchyTreeView.SelectedItem.MachineName;
            classEditorEdit.Mode = ClassEditorEdit.EditorEditMode.Create;
        }

        private void EditClassDialog()
        {
            var i = hierarchyTreeView.SelectedItem;
            if (i == null) return;

            lastSelectedBackupId = i.internalTreeId;
            ClassRecord rec = ClassManager.GetClassByMachineName(i.MachineName);
            if (rec != null)
            {
                classEditorEdit = EditorWindow.GetWindow(typeof(ClassEditorEdit), true, "Edit Class") as ClassEditorEdit;
                ModalChild = classEditorEdit;
                classEditorEdit.ClassRec = rec;
                classEditorEdit.Mode = ClassEditorEdit.EditorEditMode.Edit;
            }
        }

        private void BEdit_OnClick(object sender)
        {
            EditClassDialog();
        }

        private void LoadAttributes(string machineNameClass)
        {
            attributeTable.Clear();

            var l = PropertyManager.PropertiesOfClass(machineNameClass);

            Dictionary<string, List<PropertyBase>> list = new Dictionary<string, List<PropertyBase>>();
            foreach (var r in l)
            {
                var recDb = r.Key;
                var rec = r.Value;
                var prs = rec.PropertySerialized;
                var t = rec.PropertyType;
                var pr = PropertyBase.FromSerialized(t, prs);
                if (!list.ContainsKey(pr.Category))
                {
                    list.Add(pr.Category, new List<PropertyBase>());
                }
                list[pr.Category].Add(pr);
            }

            var categories = list.Keys.ToList();
            categories.Sort();

            foreach (var category in categories)
            {
                var batch = list[category];
                batch.Sort((x, y) => x.Name.CompareTo(y.Name));

                attributeTable.AddTitle(category);

                foreach (var pr in batch)
                {
                    PropertyInfo prInfo = (PropertyInfo)Attribute.GetCustomAttribute(pr.GetType(), typeof(PropertyInfo));
                    List<string> row = new List<string>();
                    row.Add(pr.Name);
                    row.Add(prInfo.Name);
                    string c = "[Cnt: " + (pr.ValueCount != 0 ? pr.ValueCount.ToString() : "inf") ;
                    c += "], [Exp: ";
                    c += ((pr.ExposeToTemplates) ? "T" : "-");
                    c += "/";
                    c += ((pr.ExposeToInstances) ? "I" : "-");
                    c += "]";
                    if (pr.ValuePropagation != "")
                        c += ", [Prop]";
                    row.Add(c);
                    row.Add(pr.Description);
                    attributeTable.AddRow(row);
                    attributeTable.GetLastRow().Tag = pr.MachineName;
                }
            }
        }

        private Type lastAttType = null;
        private PropertyRecord lastAttRecord = null;
        private string lastAttMachName = "";
        private bool lastAttAddmode = false;

        private void AddAttributeDialog()
        {
            if (hierarchyTreeView.SelectedItem == null)
                return;

            ClassRecord cl = ClassManager.GetClassByMachineName(hierarchyTreeView.SelectedItem.MachineName);
            if (cl == null)
                return;

            propTypeWindow = EditorWindow.GetWindow(typeof(PropertyTypeWindow), true, "Choose Attribute Type") as PropertyTypeWindow;
            ModalChild = propTypeWindow;
            propTypeWindow.ClassTitle = cl.className;
        }

        private void AddAttributeDialogAfter(Type t)
        {
            if (hierarchyTreeView.SelectedItem == null || t == null)
                return;
            ClassRecord cl = ClassManager.GetClassByMachineName(hierarchyTreeView.SelectedItem.MachineName);
            if (cl == null)
                return;

            var newPr = Activator.CreateInstance(t) as PropertyBase;
            newPr.MachineName = "";
            newPr.Name = "";
            newPr.Description = "";

            PropertyRecord emptyRecord = (PropertyRecord)ScriptableObject.CreateInstance(typeof(PropertyRecord));
            emptyRecord.PropertySerialized = newPr.ToSerialized(t.FullName);
            emptyRecord.ClassMachineName = cl.machineName;
            emptyRecord.PropertyType = t.FullName;

            lastAttType = t;
            lastAttRecord = emptyRecord;
            lastAttAddmode = true;

            PropertyWindow.ShowSettings(emptyRecord, this, "Create Attribute");
        }

        private void AddAttributeDialogFinalize(DialogResult result)
        {
            if (hierarchyTreeView.SelectedItem == null || lastAttType == null || lastAttRecord == null)
                return;
            ClassRecord cl = ClassManager.GetClassByMachineName(hierarchyTreeView.SelectedItem.MachineName);
            if (cl == null)
                return;

            if (result != DialogResult.OK)
            {
                // Destroy the empty record from the previous phase.
                ScriptableObject.DestroyImmediate(lastAttRecord);
            }
            else
            {
                // Everything should be already set. Just save it to DB, create asset...
                PropertyManager.Create(PropertyBase.FromSerialized(lastAttType.FullName, lastAttRecord.PropertySerialized), cl.machineName, lastAttType.FullName, true);
                LoadAttributes(cl.machineName);
            }
        }

        private bool SelectedClassAndAttribute(out ClassRecord classRec, out PropertyRecord propRec)
        {
            classRec = null;
            propRec = null;

            if (hierarchyTreeView.SelectedItem == null || attributeTable.SelectedRow == -1 || attributeTable.SelectedRow >= attributeTable.Rows.Count)
                return false;

            ClassRecord cl = ClassManager.GetClassByMachineName(hierarchyTreeView.SelectedItem.MachineName);
            if (cl == null)
                return false;

            classRec = cl;

            string prm = attributeTable.Rows[attributeTable.SelectedRow].Tag;
            var pr = PropertyManager.GetPropertyByMachineName(cl.machineName, prm);
            if (pr == null)
                return false;

            propRec = pr;

            return true;
        }

        private void EditAttributeDialog()
        {
            ClassRecord clRec;
            PropertyRecord prRec;
            if (!SelectedClassAndAttribute(out clRec, out prRec))
                return;

            PropertyRecord editRec = PropertyManager.ClonedProperty(prRec);

            var unserialized = PropertyBase.FromSerialized(editRec.PropertyType, editRec.PropertySerialized);

            lastAttType = Type.GetType(prRec.PropertyType);
            lastAttRecord = editRec;
            lastAttMachName = unserialized.MachineName;
            lastAttAddmode = false;

            PropertyWindow.ShowSettings(editRec, this, "Edit Attribute");
        }

        private void EditAttributeDialogFinalize(DialogResult result)
        {
            if (result != DialogResult.OK)
            {
                // Destroy the empty record from the previous phase.
                ScriptableObject.DestroyImmediate(lastAttRecord);
            }
            else
            {
                // Everything should be already set. Just save it to DB, create asset...
                ClassRecord clRec;
                PropertyRecord prRec;
                if (!SelectedClassAndAttribute(out clRec, out prRec))
                    return;

                PropertyManager.Edit(
                    PropertyBase.FromSerialized(lastAttType.FullName, lastAttRecord.PropertySerialized),
                    lastAttMachName, lastAttRecord.ClassMachineName, lastAttType.FullName, true);
                LoadAttributes(lastAttRecord.ClassMachineName);
            }
        }
    }
}