﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UPP.OGUI;
using System.Linq;
using System;
using UnityEditor;

namespace UPP.Engine
{

    public class PropertyTypeWindow : Window
    {
        private const int width = 300;
        private const int height = 150;

        ComboBox cbType;
        Label lDescription;
        public string ClassTitle = "Unknown";

        private List<KeyValuePair<string, object[]>> data = new List<KeyValuePair<string, object[]>>();

        public Type GetSelectedType()
        {
            if (cbType.SelectedIndex == -1)
                return null;
            Type t = (Type)data.Find(x => x.Key == cbType.SelectedKey).Value[3];
            return t;
        }

        private void fillCombo()
        {
            var subclasses =
                from assembly in AppDomain.CurrentDomain.GetAssemblies()
                from type in assembly.GetTypes()
                where type.IsSubclassOf(typeof(PropertyBase))
                select type;

            data.Clear();
            cbType.Clear();

            foreach (Type t in subclasses)
            {
                var prInfo = (PropertyInfo)Attribute.GetCustomAttribute(t, typeof(PropertyInfo));
                if (prInfo == null)
                    continue;

                object[] o =
                {
                    prInfo.MachineName,
                    prInfo.Name,
                    prInfo.Description,
                    t
                };
                data.Add(new KeyValuePair<string, object[]>(prInfo.MachineName, o));
                cbType.Add(prInfo.MachineName, prInfo.Name);
            }
        }


        protected override void PrepareGUI()
        {
            position = new Rect(position.x, position.y, width, height);
            minSize = new Vector2(width, height);

            Label l0 = new Label("New attribute for class " + ClassTitle, true);
            l0.Position = new Position(10, 10);

            Label l = new Label("Choose type:");
            l.Position = new Position(10, 35);

            DialogConfirmPanel p = new DialogConfirmPanel(this, position, "Create", "Cancel");
            p.OnConfirm += P_OnConfirm;

            cbType = new ComboBox("<Select>");
            cbType.Position = new Position(150, 35);
            cbType.Anchor = new Anchor(true, true, false, true);
            cbType.Size.Width = width - 155;
            fillCombo();
            cbType.OnChange += CbType_OnChange;

            lDescription = new Label("No description.");
            lDescription.Position = new Position(10, 60);
            lDescription.AutoSize = false;
            lDescription.BackgroundColor = ColorSchemes.BackgroundColorDarker;
            lDescription.Size = new Size(width - 20, height - 95);
            lDescription.Anchor = new Anchor(true, true, true, true);
        }

        private void P_OnConfirm(DialogConfirmPanel.ConfirmFeedback f)
        {
            if (cbType.SelectedIndex == -1)
            {
                EditorUtility.DisplayDialog("Not a valid Type", "Please select a Type first.", "OK");
                f.Close = false;
            }
        }

        private void CbType_OnChange(object sender)
        {
            foreach (var d in data)
            {
                if (d.Key == cbType.SelectedKey)
                {
                    lDescription.Text = d.Value[2] as string;
                    return;
                }
            }
        }
    }

}