﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEditor;
using UnityEngine;
using UPP.OGUI;

namespace UPP.Engine
{

    public class SceneTools : Window
    {
        float width = 200;
        float height = 500;

        Panel buttonHolder;

        [MenuItem("Unity++/Scene Tools")]
        public static void ShowWindow()
        {
            EditorWindow.GetWindow(typeof(SceneTools), false, "Scene Tools");
        }

        private void FillCombo(ComboBox box)
        {
            box.Clear();
            var all = ClassManager.DB.database;
            List<ClassRecord> list = new List<ClassRecord>();
            foreach (var c in all)
            {
                var cl = ClassManager.GetClassByMachineName(c.machineName);
                if (!cl.isAbstract)
                    list.Add(cl);
            }
            list.Sort((x, y) => x.className.CompareTo(y.className));
            foreach (var l in list)
            {
                box.Add(l.machineName, l.className);
            }
        }

        List<Button> buttons = new List<Button>();
        private void ReconstructButtons(ClassRecord SelectedTemplateType)
        {
            foreach (Button b in buttons)
            {
                if (b.Parent != null)
                    b.Parent.RemoveControl(b);
            }

            buttons = new List<Button>();
            
            if (SelectedTemplateType == null)
                return;

            var list = TemplateManager.FindTemplatesOfType(SelectedTemplateType.machineName);
            var cat = new List<KeyValuePair<string, List<TemplateRecord>>>();
            var cats = new List<string>();
            foreach (var l in list)
            {
                var rec = l.Value;
                if (!cats.Contains(rec.Category))
                {
                    cats.Add(rec.Category);
                    cat.Add(new KeyValuePair<string, List<TemplateRecord>>(rec.Category, new List<TemplateRecord>()));
                }
                cat.Find(x => x.Key == rec.Category).Value.Add(rec);
            }
            cat.Sort((x, y) => x.Key.CompareTo(y.Key));
            foreach (var c in cat)
            {
                var recs = c.Value;
                recs.Sort((x, y) => x.TemplateName.CompareTo(y.TemplateName));
                if (c.Key != "")
                {
                    Label l = new Label(c.Key);
                    l.Dock = DockStyle.Top;
                }

                foreach (var rec in recs)
                {
                    Button b = new Button(rec.TemplateName);
                    b.Parent = buttonHolder;
                    b.Dock = DockStyle.Top;
                    buttons.Add(b);
                    b.OnClick += (sender) => BtnClick(b, rec.TemplateMachineName);
                   // TreeView.TreeViewItem item = new TreeView.TreeViewItem(rec.TemplateMachineName, rec.TemplateName);
                }
            }
        }

        void BtnClick(object sender, string machine)
        {
            var o = new GameObject();
            string s = Settings.GetOrSet("UPP_Output_Namespace", "UPP.Game") +
                "." + machine;
            Type t = Type.GetType(s);
            o.AddComponent(t);
         /*   Selection.activeObject = SceneView.currentDrawingSceneView;
            Camera sceneCam = SceneView.currentDrawingSceneView.camera;
            Vector3 spawnPos = sceneCam.ViewportToWorldPoint(new Vector3(0.5f, 0.5f, 0f));
            o.transform.position = spawnPos;*/
        }

        protected override void PrepareGUI()
        {
            position = new Rect(position.x, position.y, width, height);

            Panel p = new Panel();
            p.Dock = DockStyle.Top;
            p.Size.Height = 50;
            p.BackgroundColor = ColorSchemes.BackgroundColorDarker;
            p.BorderColor = ColorSchemes.BorderColor;

            Label l = new Label("Template type:");
            l.Position = new Position(3, 5);
            l.Parent = p;

            buttonHolder = new Panel();
            buttonHolder.Dock = DockStyle.Fill;

            ComboBox cb = new ComboBox("<Select>");
            cb.Parent = p;
            cb.Position = new Position(0, 24);
            cb.Anchor = new Anchor(true, true, false, true);
            cb.Size.Width = width;
            FillCombo(cb);
            cb.OnChange += Cb_OnChange;
        }

        private void Cb_OnChange(object sender)
        {
            var machine = (sender as ComboBox).SelectedKey;
            var cls = ClassManager.GetClassByMachineName(machine);
            ReconstructButtons(cls);
        }
    }

}